
/*
  comlist.cpp - Windows COM Port Enumeration Utility (MinGW)

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  Reference from:

    http://yamatyuu.net/computer/program/comlist/index.html

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/

#include <windows.h>
#include <setupapi.h> // Requires a link for setupapi.dll (-lsetupapi)
#include <stdio.h>
#include <tchar.h>

#if 0 // MinGW error!!
  #include <winioctl.h>
#else
  const GUID GUID_DEVINTERFACE_COMPORT =
    { 0x86e0d1e0, 0x8089, 0x11d0, 0x9c, 0xe4, 0x08, 0x00, 0x3e, 0x30, 0x1f, 0x73 };
#endif

int comList(void)
{
  int count = 0;
  HDEVINFO hDevInfo = SetupDiGetClassDevs(&GUID_DEVINTERFACE_COMPORT, 0, 0, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
  if (hDevInfo)
  {
    SP_DEVINFO_DATA data = { sizeof(data) };
    while (SetupDiEnumDeviceInfo(hDevInfo, count++, &data))
    {
      HKEY key = SetupDiOpenDevRegKey(hDevInfo, &data, DICS_FLAG_GLOBAL, 0, DIREG_DEV, KEY_QUERY_VALUE);
      if (key)
      {
        TCHAR name[256];
        DWORD size = sizeof(name);
        RegQueryValueEx(key, TEXT("PortName"), NULL, NULL, (LPBYTE)name, &size);
        _tprintf(TEXT("%s"), name);
        if (SetupDiGetDeviceRegistryProperty(hDevInfo, &data, SPDRP_DEVICEDESC, NULL, (LPBYTE)name, sizeof(name), NULL))
          _tprintf(TEXT(", %s"), name);
        else
          _tprintf(TEXT(", -"));
        if (SetupDiGetDeviceRegistryProperty(hDevInfo, &data, SPDRP_HARDWAREID, NULL, (LPBYTE)name, sizeof(name), NULL))
          _tprintf(TEXT(", %s\n"), name);
        else
          _tprintf(TEXT("\n"));
      }
    }
    SetupDiDestroyDeviceInfoList(hDevInfo);
  }
  return count;
}

int main(void)
{
  return comList();
}
